//+------------------------------------------------------------------+
//|                                                    SocketTLS.mqh |
//|                                  Copyright 2022, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| TLS Socket                                                       |
//+------------------------------------------------------------------+
class CSocketTLS
  {
private:
   int               m_socket;

public:
                     CSocketTLS(void): m_socket(INVALID_HANDLE)
     {
     }

                    ~CSocketTLS(void)
     {
      Close();
     }

   bool              Connect(string server,int port=443,int timeout=3000)
     {
      //--- reopen socket
      Close();

      m_socket=SocketCreate();
      if(m_socket==INVALID_HANDLE)
         return(false);
      //--- connect
      return SocketConnect(m_socket,server,port,timeout);
     }

   void               Close(void)
     {
      if(m_socket!=INVALID_HANDLE)
        {
         SocketClose(m_socket);
         m_socket=INVALID_HANDLE;
        }
     }

   bool              Send(string request)
     {
      //--- check
      if(m_socket==INVALID_HANDLE)
         return(false);
      //--- convert to array
      char reqbuf[];
      int  reqlen=StringToCharArray(request,reqbuf,0,WHOLE_ARRAY,CP_UTF8);

      if(reqlen<1)
        {
         Close();
         return(false);
        }
      //--- send without last zero
      if(SocketTlsSend(m_socket,reqbuf,reqlen-1)!=reqlen-1)
         return(false);
      //---
      return(true);
     }

   int               Read(char &answer[],ulong timeout=120000)
     {
      //--- check
      if(m_socket==INVALID_HANDLE)
         return(-1);
      //---
      uint  available;
      char  buffer[];
      int   buffer_size=0;
      int   answer_size=0;
      ulong expiration=GetTickCount64()+timeout;

      ArrayResize(buffer,16384);
      do
        {
         //--- check availability of raw data
         if((available=SocketIsReadable(m_socket))<1)
           {
            Sleep(5);
            continue;
           }
         //--- multiple reads
         do
           {
            //--- read portion and check
            buffer_size=SocketTlsReadAvailable(m_socket,buffer,16384);

            if(buffer_size==-1)
               if(GetLastError()==ERR_NETSOCKET_IO_ERROR)
                 {
                  Close();
                  return(answer_size);
                 }
            //--- add to the answer buffer
            if(buffer_size>0)
              {
               ArrayInsert(answer,buffer,answer_size,0,buffer_size);
               answer_size+=buffer_size;
               expiration=GetTickCount64()+timeout;
              }
           }
         while(buffer_size>0);
         //--- data is avavailable?
         if(answer_size>0)
            return(answer_size);
        }
      while(!IsStopped() && GetTickCount64()<expiration);
      //--- no data
      return(0);
     }

   string            ReadString(ulong timeout=120000)
     {
      char answer[];
      int  answer_size=Read(answer,timeout);
      //--- check
      if(answer_size<1)
         return("");

      return CharArrayToString(answer);
     }

  };
//+------------------------------------------------------------------+
